/* Modification made in the beginning of May, 2000:
   now Detector class defined below is not used. The HPT_Setup class is used
   directly instead.
   The Detector was intended to store different Setups (probably of different
   types, you may read below about it). Currently only one HPT_Setup is used,
   so there is no need to put it alone in the container.
   */

/* Defines Detector class as a container of Setup's of different 
   types. 
   ----------------------------------------------------------------------
   What should be changed when new Setup type is created:
   add the check:
      else if ( psetup->type() == Your_Setup::setup_type() )
        action(*((Your_Setup*)psetup));
   in 
      void ApplyVirtually<Action>::operator()(__SetupBase *psetup) {
        ...
      }
   (see below).
   ----------------------------------------------------------------------
   Detector == map<string, PtrWrapper<__SetupBase>, less<string> >
   i.e. Setups are included via the PtrWrapper of the base class
   __SetupBase.

   To find out the actual type of the Setup one needs to make an 
   explicit cast from __SetupBase. The standard way to do it is to use
   RTTI (Run Time Type Information) implemented
   in C++. But many compilers do not support this feature. Therefore
   this casting is made "by hand". Of course standard way would be more 
   efficient.
   
   The type of Setup is determined using 
     virtual const string& __SetupBase::type() const;
   which returns unique string for each of possible Setup types derived
   from __SetupBase. 
   There is ApplyVirtually<Action> class. It has 
      void operator()(__SetupBase *psetup);
   operator. This is the only place where the casting is done explicitely
   using psetup->type() value. If the string psetup->type() is unknown
   the global_error() is set (see global_error.hh).
   
   If you create new Setup which should be included in the Detector
   you should add one more 

     else if ( psetup->type() == Your_Setup::setup_type() )
       action(*((Your_Setup*)psetup));

   case in this operator. This makes an explicit cast from __SetupBase
   to Your_Setup type. This is the only place to be changed. If someone
   needs to make a cast in his code this should be done via 
   ApplyVirtually<Action>. Actually it is convinient to use the function
     apply_virtually(action);
   it just returns ApplyVirtually<Action>(action) where the Action is the
   type of action.
   Action should be able to work with all Setup types in the Detector.
   That is the expressions 
     action(any_setup);
   should be valid, where any_setup can be of any Setup types.
   When apply_virtually(action) is applied to some __SetupBase* psetup:
     apply_virtually(action)(psetup);
   the action is redirected to 
     action(real_setup);
   where real_setup is the actual Setup (not Setup*) determined using
   psetup.
   E.g. 
   struct PrintNumberOfChambers {
     template <class AnySetup>
     void operator()(AnySetup& setup) {
       cout << setup.size();
     }
   };
   for (Detector::iterator iter=detector.begin(); iter!=detector.end();
        ++iter) {
     cout << "Setup " << (*iter).first << ": ";
     apply_virtually(PrintNumberOfChambers())( ((*iter).second)() );
     cout << '\n';
   }

   Here the template member functions (like 
     void operator()(AnySetup& setup);
   in PrintNumberOfChambers)
   can be quite useful if your "action" is general enough to be applied
   to any setup. E.g. if it depends only on the "geometrical" aspects
   of the Setup which are the same for all Setups. 
   If it is not the case one may define function object with a set
   of explicit 
     void operator()(Setup1& s1) { ... }
     void operator()(Setup2& s2) { ... }
     void operator()(Setup3& s3) { ... }
   for every Setup,
   or it can be convenient to define such operator's for some Setups
   while for "all others" define the template <class AnyOtherSetup>
   operator (default action).

   In addition to 
     apply_virtually(action)(psetup);
   where psetup is of type __SetupBase*, one may use 
     apply_virtually(action)(detector, setup_name);
   Here the psetup is determined using detector container and setup_name
   key. If the setup with setup_name is not found in detector the
   global_error() is set. Thus one can write the above printing loop as:

   for (Detector::iterator iter=detector.begin(); iter!=detector.end();
        ++iter) {
     const string& setup_name = (*iter).first;
     cout << "Setup " << setup_name << ": ";
     apply_virtually(PrintNumberOfChambers())(detector, setup_name);
     cout << '\n';
   }
----------------------------------------------------------------------
   To iterate through all the cells (sections, chambers, setups) in
   the detector on may use for_each_...(action, detector);
   which are defined in etector_tools.hh.
----------------------------------------------------------------------
   */

// Author V.Balagura (balagura@mail.desy.de)   (17.02.99).

#ifndef _detector_hh
#define _detector_hh 1

#include "ptset/setup.hh"
#include "ptset/high_pt.hh"
#include "ptset/global_error.hh"

class Detector : 
public map<string, PtrWrapper<__SetupBase>, less<string> > {
public:
  string name;

  Detector() {}
  Detector(const char* detector_name) { name = detector_name; }
};

/* --------------------apply_virtually(action) -------------------- */
template <class Action>
class ApplyVirtually {
  Action action;
public:
  ApplyVirtually(const Action& act) : action(act) {}

  void operator()(__SetupBase *psetup) {
    if ( psetup->type() == HPT_Setup::setup_type() )
      action(*((HPT_Setup*)psetup));
    // any other Setups here ...
    else {
      /* apply(x, *psetup); */
      global_error() << ("unrecognized setup type: \"" + psetup->type()
			 + "\"");
    }
  }
  void operator()(Detector& detector, string setup_name) {
    Detector::iterator iter;
    if ( (iter = detector.find(setup_name)) == detector.end() ) {
      global_error()("there is no setup \"" + setup_name
		     + "\" in detector \"" + detector.name + "\"");
      return;
    }
    operator()( ((*iter).second)() );
  }
};

template <class Action>
inline ApplyVirtually<Action> apply_virtually(const Action& action) {
  return ApplyVirtually<Action>(action);
}

#endif
